/***********************************************************************/
/*                                                                     */
/*  FILE        :ICS_sample.c                                          */
/*  DATE        :Sep, 8, 2022                                          */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :RX26T                                                 */
/*                                                                     */
/*  NOTE:THIS IS AN ICS++ EXAMPLE.                                     */
/*                                                                     */
/***********************************************************************/

#ifdef  __ICCRX__
  #include "r_cg_macrodriver.h"
  #include <math.h>
#elif   __CCRX__
  #include <machine.h>
  #include <mathf.h>
  #include <stdint.h>
  #include "iodefine.h"
#else
  //
#endif

#include "ICS2_RX26T.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Set option bytes */

#ifdef   __ICCRX__

  #if __LITTLE_ENDIAN__
    #pragma public_equ="__MDES",0xFFFFFFFFUL
  #else
    #pragma public_equ="__MDES",0xFFFFFFF8UL
  #endif

  #pragma public_equ="__OFS0",0xFFFFFFFFUL
  #pragma public_equ="__OFS1",0xFFFFFFFFUL

#elif   __CCRX__  
  
  
#endif


#define   TWOPI    (2.0f*3.1415926535f)
#define   BRR    0     //  HOCO=16MHz PLL=x15 = CPUCLK=120MHz, BRR=0, ICS=7.5Mbps, PCLKB=60MHz
                       //  BRR  BPS         DTLclock 
                       //   0  60/ 8M=7.50M   60MHz  
                       //   1  60/16M=3.75M   30MHz  
                       //   2  60/24M=2.50M   20MHz
                       //   3  60/32M=1.875M  15MHz    
                       //   4  60/40M=1.50M   12MHz
		       //   5  60/48M=1.25M   10MHz
		       
#define   MODE   3     // 0:16bit 8ch,one time (Can not use)
                       // 1:32bit 8ch,2 times
                       // 2:32bit 4ch,one time
                       // 3:32bit 12ch 3 times


#ifdef  __ICCRX__
  #pragma location=0x0F000
  __no_init  uint32_t dtc_table[256]; /* alignment 0x000 */
#elif   __CCRX__
  #pragma address dtc_table=0x0F000
  uint32_t dtc_table[256];            /* alignment 0x000 */
#else
#endif

float   wt    = 0.0f;
float   dwt   = TWOPI*50.0f/10000.0f;
float   refu  = 0.0f;
float   refv  = 0.0f;
float   refw  = 0.0f;
float   m     = 1.0f;
float   dm    = 0.0002;
int32_t up    = 0;



void main(void)
{
    SYSTEM.PRCR.WORD = 0xA50B;
    SYSTEM.OSTDCR.BYTE = 0x00;

    SYSTEM.SCKCR3.BIT.CKSEL = 0;     // Set system clock = LOCO
    SYSTEM.HOCOCR.BYTE  = 1;         // HOCO stop
    SYSTEM.HOCOCR2.BYTE = 0;         // HOCO = 16MHz  // OK
  //SYSTEM.HOCOCR2.BYTE = 2;         // HOCO = 20MHz  // 
    SYSTEM.HOCOCR.BYTE  = 0;         // HOCO start
    while (SYSTEM.OSCOVFSR.BIT.HCOVF==0) { /* no code */ }  // Wait until HOCO stable

    SYSTEM.SCKCR3.BIT.CKSEL = 1;     // Set system clock = HOCO

    SYSTEM.SCKCR.LONG  = 0x21011212; // FCK=1/4, ICK=1/2, BCLK output stop, BCK=1/4
                                     // FCK  =1/4 =  60MHz            (Max 60MHz)
                                     // ICK  =1/2 = 120MHz            (Max120MHz)
                                     // PCLKA=1/2 = 120MHz, RSCI,MTU,GPT (Max120MHz)
                                     // PCLKB=1/4 =  60MHz, others    (Max 60MHz)
                                     // PCLKC=1/2 = 120MHz, MTU,GPT   (Max120MHz)
                                     // PCLKD=1/4 =  60MHz, AD        (Max 60MHz)

    SYSTEM.PLLCR.BIT.PLIDIV    = 0;  // PLL input is divided by 1
    SYSTEM.PLLCR.BIT.PLLSRCSEL = 1;  // PLL source is HOCO
  //SYSTEM.PLLCR.BIT.STC       = 23; // PLL mul rate is x12  //
    SYSTEM.PLLCR.BIT.STC       = 29; // PLL mul rate is x15  //
    SYSTEM.PLLCR2.BYTE         = 0;  // PLL on
    while (SYSTEM.OSCOVFSR.BIT.PLOVF==0) { /* no code */ } // Wait until PLL stable
    
    SYSTEM.SCKCR3.BIT.CKSEL = 4;     // Set system clock = PLL (16MHzx15=240MHz)

  //******************************************************************************************************
                                                                      // T5402
                                                                      //  100pin  
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PD4_PD6, 6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)   
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PD7_PE0,  6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P01_P00, 6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI6_P81_P80,  6, BRR, MODE); //   OK  7.5Mbps (ICSclk=60MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P81_P80, 6, BRR, MODE); //   OK  7.5Mbps (ICSclk=60MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P23_P22, 6,   1, MODE); //   OK 3.75Mbps (ICSclk=60MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI6_PB2_PB1,  6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PB5_PB6,  6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PB5_PB6, 6,   1, MODE); //   OK 3.75Mbps (ICSclk=30MHz)     

  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6,   2, MODE); //   OK 2.50Mbps (ICSclk=20MHz) 
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6,   3, MODE); //   OK 1.875Mbps(ICSclk=15MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6,   4, MODE); //   OK 1.50Mbps (ICSclk=12MHz)
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6,   5, MODE); //   OK 1.25Mbps (ICSclk=10MHz)
  
    ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PD4_PD6, 6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)   
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PD7_PE0,  6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P01_P00, 6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI6_P81_P80,  6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P81_P80, 6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P23_P22, 6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)  
  //ics2_init((void *)(&dtc_table), ICS_SCI6_PB2_PB1,  6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PB5_PB6,  6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)     
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PB5_PB6, 6, 251, MODE); //   OK    1Mbps (ICSclk= 8MHz)     
  
  
  //******************************************************************************************************

#ifdef  __ICCRX__
    EI();
#elif   __CCRX__
    set_ipl(0); /* set lowest IPL level */
    setpsw_i();  // enable interrput
#else
    //
#endif

    while (1)
    {
        int32_t i;
        
        wt = wt + dwt;
        if (wt>TWOPI) { wt = wt - TWOPI; }
        
        if (up==1)
        {
             m = m + dm;
             if (m>=1.0f) { up = 0; }
        }
        else
        {
             m = m - dm;
             if (m<=0.5f) { up = 1; }
        }

        refu  = m * sinf(wt);
        refv  = m * sinf(wt-TWOPI*1.0f/3.0f);
        refw  = m * sinf(wt-TWOPI*2.0f/3.0f);

        for(i=0;i<10000;i++) { nop(); }

        ics2_watchpoint();
    }
}

